import * as React from "react";
import { tween } from "./tween";
class SlideBox extends React.PureComponent {
    constructor(props) {
        super(props);
        this.slideDom = React.createRef();
        this.timer = 0;
        this.firstSlide = this.props.slide;
        this.arr = [];
        this.state = {};
        this.slideFn = props.isImmedia ? this.immediaFn : this.queue;
    }
    queue() {
        this.arr.push(this.props.slide);
        if (!this.timer) {
            const slide = this.arr.shift();
            this.queueFn(slide);
        }
    }
    componentDidUpdate(preProps) {
        if (preProps.slide !== this.props.slide) {
            this.slideFn(this.props.slide);
        }
    }
    queueFn(slideDown) {
        const { directionUp, duration } = this.props;
        const dom = this.slideDom.current;
        if (slideDown) {
            dom.style.display = "block";
            dom.style.height = "0px";
        }
        const child = dom.firstElementChild;
        const origin = child.clientHeight;
        let timeFrom = 0;
        const timeEnd = Math.ceil(duration / 12);
        const dispaly = slideDown ? "block" : "none";
        const start = slideDown ? 0 : origin;
        const end = !slideDown ? 0 : origin;
        const direct = directionUp ? "top" : "bottom";
        child.style.position = "absolute";
        const originW = child.style.width;
        child.style.width = "100%";
        child.style[direct] = "0";
        const fn = () => {
            this.timer = window.requestAnimationFrame(() => {
                const h = tween.easeInOutSine(timeFrom, start, end, timeEnd);
                timeFrom++;
                dom.style.height = `${h}px`;
                if (timeFrom < timeEnd) {
                    fn();
                }
                else {
                    dom.style.display = dispaly;
                    dom.style.height = "";
                    this.timer = 0;
                    if (this.arr.length) {
                        const newSlide = this.arr.shift();
                        this.queueFn(newSlide);
                    }
                    child.style.position = "";
                    child.style[direct] = "";
                    child.style.width = originW;
                }
            });
        };
        fn();
    }
    immediaFn(slideDown) {
        const { directionUp } = this.props;
        const dom = this.slideDom.current;
        const hasRun = !!this.timer;
        if (hasRun) {
            window.cancelAnimationFrame(this.timer);
            this.timer = 0;
        }
        if (slideDown && !hasRun) {
            dom.style.display = "block";
            dom.style.height = "0px";
        }
        const child = dom.firstElementChild;
        const origin = child.clientHeight;
        let start;
        const duration = 300;
        let timeFrom = 0;
        const timeEnd = Math.ceil(duration / 17);
        if (hasRun) {
            start = parseInt(dom.style.height, 0);
        }
        else {
            start = slideDown ? 0 : origin;
        }
        const end = !slideDown ? 0 : origin;
        const dispaly = slideDown ? "block" : "none";
        const direct = directionUp ? "top" : "bottom";
        const originW = child.style.width;
        child.style.position = "absolute";
        child.style[direct] = "0";
        child.style.width = "100%";
        const fn = () => {
            this.timer = window.requestAnimationFrame(() => {
                const h = tween.linear(timeFrom, start, end, timeEnd);
                timeFrom++;
                dom.style.height = `${h}px`;
                if (timeFrom < timeEnd) {
                    fn();
                }
                else {
                    dom.style.height = "";
                    dom.style.display = dispaly;
                    this.timer = 0;
                    child.style.position = "";
                    child.style[direct] = "";
                    child.style.width = originW;
                }
            });
        };
        fn();
    }
    render() {
        const { children, className, styleObj } = this.props;
        const style = Object.assign({ display: this.firstSlide ? "block" : "none", }, styleObj);
        return (React.createElement("div", { ref: this.slideDom, className: `g-slideBox ${className || ""}`, style: style }, children));
    }
}
SlideBox.defaultProps = {
    duration: 300
};
export default SlideBox;
